// Define the table and column name for the list of roles
const string roleTableName = "Roles";
const string roleColumnName = "Role";
const string adminName = "Admin";

const string measureIsPrefix = "Is ";
const string measureIsSuffix = "";
const string activeRoleMeasureName = "Active Role";
const string activeRolesMeasureName = "Active Roles";


// Stop if there are no roles
if (Model.Roles.Count < 1) {
    throw new Exception( "No roles defined in the model");
}

// Create list of role names + Admin
var roleNames = Model.Roles.Select( r => r.Name ).ToList();
if (!string.IsNullOrWhiteSpace(adminName)) {
    if (roleNames.Contains(adminName)) {
        throw new Exception( "Role {adminName} is in conflict with the special adminName constant.");
    }   
    roleNames.Add(adminName);
}

// Create the table with the roles if it doesn't exist, or update the DAX code if it exists
string listRoleNames = roleNames.Skip(1).Aggregate("\"" + roleNames[0] + "\"", (current, r) => current + ", \"" + r + "\"");
string roleTableDax = "SELECTCOLUMNS ( { " + listRoleNames + " }, \"" + roleColumnName + "\", [Value] )";
var roleTable = Model.Tables.FindByName(roleTableName);
if (roleTable == null) {
    roleTable = Model.AddCalculatedTable(roleTableName,roleTableDax);
} 
else {
    (roleTable as CalculatedTable).Expression = roleTableDax;
}

// Apply security to existing roles
foreach ( var role in Model.Roles ) {
    role.RowLevelSecurity[roleTable] = roleTableName + "[" + roleColumnName + "] == \"" + role.Name + "\"";
}

// Create measures to test active roles + Admin
foreach ( var roleName in roleNames ) {
    string measureName = measureIsPrefix + roleName + " " + roleColumnName + measureIsSuffix;
    string measureDax = "\"" + roleName + "\" IN VALUES ( " + roleTableName + "[" + roleColumnName + "] )";
    var measure = roleTable.Measures.FindByName(measureName) ?? roleTable.AddMeasure( measureName, measureDax );
    measure.IsHidden = true;
}

// Create measures for Active Role and Active Roles
string activeRoleMeasureDax = "SELECTEDVALUE ( " + roleTableName + "[" + roleColumnName + "] )";
var measureActiveRole = roleTable.Measures.FindByName(activeRoleMeasureName) ?? roleTable.AddMeasure( activeRoleMeasureName, activeRoleMeasureDax );
string activeRolesMeasureDax = "CONCATENATEX ( VALUES ( " + roleTableName + "[" + roleColumnName + "] ), " + roleTableName + "[" + roleColumnName + "], \", \" )";
var measureActiveRoles = roleTable.Measures.FindByName(activeRolesMeasureName) ?? roleTable.AddMeasure( activeRolesMeasureName, activeRolesMeasureDax );

// Hide table and active role measures - comment or change if they should be visible
roleTable.IsHidden = true;
measureActiveRole.IsHidden = true;
measureActiveRoles.IsHidden = true;
